//#include <FS.h>  ;// placer au début de tout
#include <ESP8266WebServer.h>
#include <WiFiManager.h>
#include <ArduinoJson.h>
/*
  char mqtt_server[40]             = "url du serveur.........................";
  char mqtt_server_defaut[40]      = "url_serveur";
  char mqtt_port[6]                = "1883";
  char mqtt_port_defaut[6]         = "1883";
  char client_mqtt[20]             = "nom du capteur....";
  char client_mqtt_defaut[20]      = "nom_du_capteur";
  char user_mqtt[20]               = "utilisateur.......";
  char user_mqtt_defaut[20]        = "utilisateur";
  char user_pass[20]               = "mot de passe......";
  char user_pass_defaut[20]        = "mot_de_passe";
*/
//flag for saving data
bool shouldSaveConfig = false; // true si action WifiManager
void printFile(const char *filename) {
  File file = SPIFFS.open(filename, "r");
  if (!file) {
    Serial.println(F("Failed to read file"));
    return;
  }
  while (file.available()) {
    Serial.print((char)file.read());
  }
  Serial.println();
  file.close();
}
//callback notifying us of the need to save config
void saveConfigCallback () {
  Serial.println("Should save config");
  shouldSaveConfig = true;
}
void WifiManager_UI() {
  //clean FS, for testing
  //SPIFFS.format();
  if (SPIFFS.begin()) {
    Serial.println("mounted file system");
    if (SPIFFS.exists("/config.json")) {
      ////lecture config json ///
      Serial.println("reading config file");
      File configFile = SPIFFS.open("/config.json", "r");
      if (configFile) {
        printFile("/config.json");

        Serial.println("opened config file");
        StaticJsonDocument<1024> json;
        DeserializationError error = deserializeJson(json, configFile);
        if (!error) {
          strlcpy(mqtt_server, json["mqtt_server"] | mqtt_server_defaut , sizeof(mqtt_server) );
          strlcpy(mqtt_port  , json["mqtt_port"]   | mqtt_port_defaut   , sizeof(mqtt_port) );
          strlcpy(user_mqtt  , json["user_mqtt"]   | user_mqtt_defaut   , sizeof(user_mqtt) );
          strlcpy(user_pass  , json["user_pass"]   | user_pass_defaut   , sizeof(user_pass));
          strlcpy(client_mqtt, json["client_mqtt"] | client_mqtt_defaut , sizeof(client_mqtt) );

          Serial.println("json lu");
        } else {
          Serial.println("json pb");
          strncpy(mqtt_server, mqtt_server_defaut, 40 );
          strncpy(user_mqtt  , user_mqtt_defaut  , 20 );
          strncpy(user_pass  , user_pass_defaut  , 20 );
          strncpy(mqtt_port  , mqtt_port_defaut  , 6 );
          strncpy(client_mqtt, client_mqtt_defaut, 20 );
        }
      } else {
        Serial.println("failed to load json config");
      }
      configFile.close();
      printFile("/config.json");
    }
  } else {
    Serial.println("failed to mount FS");
    //clean FS, for testing
    SPIFFS.format();
    delay(1000);
    ESP.reset();
  }
  Serial.println("FS ok");
  //WiFiManager
  WiFiManagerParameter custom_mqtt_server("serveur", "url serveur" , mqtt_server, 40);
  WiFiManagerParameter custom_mqtt_port  ("port"   , "port mqtt"   , mqtt_port  , 6);
  WiFiManagerParameter custom_user_mqtt  ("uti"    , "utilsateur"   , user_mqtt  , 20);
  WiFiManagerParameter custom_user_pass  ("pass"   , "mot de passe"   , user_pass  , 20);
  WiFiManagerParameter custom_client_mqtt("client" , "client mqtt" , client_mqtt, 20);

  WiFiManager wifiManager;
  //wifiManager.setTimeout(180);
  wifiManager.setSaveConfigCallback(saveConfigCallback);

  wifiManager.addParameter(&custom_mqtt_server);
  wifiManager.addParameter(&custom_mqtt_port);
  wifiManager.addParameter(&custom_user_mqtt);
  wifiManager.addParameter(&custom_user_pass);
  wifiManager.addParameter(&custom_client_mqtt);
  if (!wifiManager.autoConnect()) {
    Serial.println("failed to connect and hit timeout");
    delay(3000);
    //reset and try again, or maybe put it to deep sleep
    ESP.reset();
    delay(5000);
  }
  Serial.println("local ip");
  Serial.println(WiFi.localIP());

  //save the custom parameters to FS
  if (shouldSaveConfig) {
    if (custom_mqtt_server.getValue()) strlcpy(mqtt_server, custom_mqtt_server.getValue(), sizeof(mqtt_server) );
    else strlcpy(mqtt_server, mqtt_server_defaut, sizeof(mqtt_server) );

    if (custom_mqtt_port.getValue())   strlcpy(mqtt_port  , custom_mqtt_port.getValue()  , sizeof(mqtt_port)  );
    else strlcpy(mqtt_port, mqtt_port_defaut, sizeof(mqtt_port));

    if (custom_user_mqtt.getValue() )  strlcpy(user_mqtt  , custom_user_mqtt.getValue()  , sizeof(user_mqtt) );
    else strlcpy(user_mqtt, user_mqtt_defaut, sizeof(user_mqtt) );

    if (custom_user_pass.getValue() )  strlcpy(user_pass  , custom_user_pass.getValue()  , sizeof(user_pass) );
    else strlcpy(user_pass, user_pass_defaut, sizeof(user_pass) );

    if (custom_client_mqtt.getValue()) strlcpy(client_mqtt, custom_client_mqtt.getValue(), sizeof(client_mqtt) );
    else strlcpy(client_mqtt, client_mqtt_defaut, sizeof(client_mqtt) );

    Serial.println("saving ");
    StaticJsonDocument<1024> json;
    json["mqtt_server"] = mqtt_server;
    json["mqtt_port"]   = mqtt_port;
    json["user_mqtt"]   = user_mqtt;
    json["user_pass"]   = user_pass;
    json["client_mqtt"] = client_mqtt;
    File configFile = SPIFFS.open("/config.json", "w");
    if (!configFile) {
      Serial.println("failed to open config file for writing");
    }
    // Serialize JSON to file
    if (serializeJson(json, configFile) == 0) {
      Serial.println(F("Failed to write to file"));
    }
    configFile.close();
  }
  printFile("/config.json");
}
